@@ -8,9 +8,13 @@
 #include <libgen.h>
 #include <errno.h>
 #include <sys/types.h>
+#include <sys/ioctl.h>
+#include <ctype.h>
+#include <dirent.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <sys/mman.h>
+#include <sys/user.h>
 #include <linux/vfio.h>
 #include <accfg/libaccel_config.h>
 #include <accfg/idxd.h>
@@ -20,6 +24,7 @@
 #include "algorithms/iaa_zcompress.h"
 #include "algorithms/iaa_compress.h"
 #include "algorithms/iaa_filter.h"
+#include "algorithms/iaa_crypto.h"
 
 static struct iaa_filter_aecs_t iaa_filter_aecs = {
 	.rsvd = 0,
@@ -46,6 +51,65 @@ static int init_crc64(struct task *tsk, int tflags, int opcode, unsigned long sr
 	return ACCTEST_STATUS_OK;
 }
 
+static int init_zcompress8(struct task *tsk, int tflags, int opcode, unsigned long src1_xfer_size)
+{
+	tsk->pattern = 0x98765432abcdef01;
+	tsk->opcode = opcode;
+	tsk->test_flags = tflags;
+	tsk->xfer_size = src1_xfer_size;
+
+	tsk->src1 = aligned_alloc(ADDR_ALIGNMENT, src1_xfer_size);
+	if (!tsk->src1)
+		return -ENOMEM;
+	iaa_zcompress16_randomize_input(tsk->src1, tsk->pattern, src1_xfer_size);
+
+	tsk->dst1 = aligned_alloc(ADDR_ALIGNMENT, IAA_ZCOMPRESS_MAX_DEST_SIZE);
+	if (!tsk->dst1)
+		return -ENOMEM;
+	memset_pattern(tsk->dst1, 0, IAA_ZCOMPRESS_MAX_DEST_SIZE);
+
+	tsk->output = aligned_alloc(ADDR_ALIGNMENT, IAA_ZCOMPRESS_MAX_DEST_SIZE);
+	if (!tsk->output)
+		return -ENOMEM;
+	memset_pattern(tsk->output, 0, IAA_ZCOMPRESS_MAX_DEST_SIZE);
+
+	tsk->iaa_max_dst_size = IAA_ZCOMPRESS_MAX_DEST_SIZE;
+
+	return ACCTEST_STATUS_OK;
+}
+
+static int init_zdecompress8(struct task *tsk, int tflags, int opcode, unsigned long input_size)
+{
+	tsk->pattern = 0x98765432abcdef01;
+	tsk->opcode = opcode;
+	tsk->test_flags = tflags;
+
+	tsk->input = aligned_alloc(ADDR_ALIGNMENT, input_size);
+	if (!tsk->input)
+		return -ENOMEM;
+	iaa_zcompress16_randomize_input(tsk->input, tsk->pattern, input_size);
+
+	tsk->src1 = aligned_alloc(ADDR_ALIGNMENT, IAA_ZDECOMPRESS_MAX_DEST_SIZE);
+	if (!tsk->src1)
+		return -ENOMEM;
+	memset_pattern(tsk->src1, 0, IAA_ZDECOMPRESS_MAX_DEST_SIZE);
+	tsk->xfer_size = iaa_do_zcompress8(tsk->src1, tsk->input, input_size);
+
+	tsk->dst1 = aligned_alloc(ADDR_ALIGNMENT, IAA_ZDECOMPRESS_MAX_DEST_SIZE);
+	if (!tsk->dst1)
+		return -ENOMEM;
+	memset_pattern(tsk->dst1, 0, IAA_ZDECOMPRESS_MAX_DEST_SIZE);
+
+	tsk->output = aligned_alloc(ADDR_ALIGNMENT, IAA_ZDECOMPRESS_MAX_DEST_SIZE);
+	if (!tsk->output)
+		return -ENOMEM;
+	memset_pattern(tsk->output, 0, IAA_ZDECOMPRESS_MAX_DEST_SIZE);
+
+	tsk->iaa_max_dst_size = IAA_ZDECOMPRESS_MAX_DEST_SIZE;
+
+	return ACCTEST_STATUS_OK;
+}
+
 static int init_zcompress16(struct task *tsk, int tflags, int opcode, unsigned long src1_xfer_size)
 {
 	tsk->pattern = 0x98765432abcdef01;
@@ -492,6 +556,173 @@ static int init_expand(struct task *tsk, int tflags,
 	return ACCTEST_STATUS_OK;
 }
 
+static int init_transl_fetch(struct task *tsk, int tflags, int opcode, unsigned long src1_xfer_size)
+{
+	tsk->src1 = aligned_alloc(PAGE_SIZE, src1_xfer_size);
+	tsk->opcode = opcode;
+	tsk->test_flags = tflags;
+	tsk->xfer_size = src1_xfer_size;
+	memset_pattern(tsk->src1, 0x0123456789abcdef, src1_xfer_size);
+	madvise(tsk->src1, src1_xfer_size, MADV_DONTNEED);
+
+	return ACCTEST_STATUS_OK;
+}
+
+static int init_encrypto(struct task *tsk, int tflags, int opcode, unsigned long src1_xfer_size)
+{
+	int i, key_size;
+	struct iaa_crypto_aecs_t *iaa_crypto_aecs;
+
+	if (tsk->crypto_aecs.algorithm != IAA_AES_CFB) {
+		err("Unsupported crypto mode %d\n", tsk->crypto_aecs.algorithm);
+		return -EPERM;
+	}
+
+	tsk->pattern = 0x98765432abcdef01;
+	tsk->opcode = opcode;
+	tsk->test_flags = tflags;
+	tsk->xfer_size = src1_xfer_size;
+
+	tsk->src1 = aligned_alloc(ADDR_ALIGNMENT, src1_xfer_size);
+	if (!tsk->src1)
+		return -ENOMEM;
+	memset_pattern(tsk->src1, tsk->pattern, src1_xfer_size);
+
+	tsk->src2 = aligned_alloc(ADDR_ALIGNMENT, IAA_CRYPTO_SRC2_SIZE);
+	if (!tsk->src2)
+		return -ENOMEM;
+	memset_pattern(tsk->src2, 0, IAA_CRYPTO_SRC2_SIZE);
+	tsk->iaa_src2_xfer_size = IAA_CRYPTO_AECS_SIZE;
+	iaa_crypto_aecs = (struct iaa_crypto_aecs_t *)tsk->src2;
+	iaa_crypto_aecs->crypto_algorithm = tsk->crypto_aecs.algorithm;
+	iaa_crypto_aecs->crypto_flags = tsk->crypto_aecs.flags;
+
+	if (iaa_crypto_aecs->crypto_flags & IAA_CRYPTO_MASK_KEY_SIZE)
+		key_size = 256;
+	else
+		key_size = 128;
+
+	iaa_crypto_aecs->crypto_flags |= IAA_CRYPTO_MASK_FLUSH_CRYPTO_IN_ACCUM;
+
+	switch (iaa_crypto_aecs->crypto_algorithm) {
+	case IAA_AES_CFB:
+		if (key_size == 256) {
+			for (i = 0; i < 4; i++)
+				iaa_crypto_aecs->aes_key_low[i] = (uint32_t)get_random_value();
+			for (i = 0; i < 4; i++)
+				iaa_crypto_aecs->aes_key_high[i] = (uint32_t)get_random_value();
+			for (i = 0; i < 4; i++)
+				iaa_crypto_aecs->counter_iv[i] = (uint32_t)get_random_value();
+		} else {
+			for (i = 0; i < 4; i++)
+				iaa_crypto_aecs->aes_key_low[i] = (uint32_t)get_random_value();
+			for (i = 0; i < 4; i++)
+				iaa_crypto_aecs->counter_iv[i] = (uint32_t)get_random_value();
+		}
+		break;
+	}
+
+	iaa_crypto_aecs->complement[8] = 1;
+	tsk->dst1 = aligned_alloc(ADDR_ALIGNMENT, src1_xfer_size);
+	if (!tsk->dst1)
+		return -ENOMEM;
+	memset_pattern(tsk->dst1, 0, src1_xfer_size);
+
+	tsk->output = aligned_alloc(ADDR_ALIGNMENT, src1_xfer_size);
+	if (!tsk->output)
+		return -ENOMEM;
+	memset_pattern(tsk->output, 0, src1_xfer_size);
+
+	tsk->iaa_max_dst_size = src1_xfer_size;
+
+	return ACCTEST_STATUS_OK;
+}
+
+static int init_decrypto(struct task *tsk, int tflags, int opcode, unsigned long input_size)
+{
+	int i, key_size;
+	struct iaa_crypto_aecs_t *iaa_crypto_aecs;
+
+	if (tsk->crypto_aecs.algorithm != IAA_AES_CFB) {
+		err("Unsupported crypto mode %d\n", tsk->crypto_aecs.algorithm);
+		return -EPERM;
+	}
+
+	tsk->pattern = 0x98765432abcdef01;
+	tsk->opcode = opcode;
+	tsk->test_flags = tflags;
+
+	tsk->input = aligned_alloc(ADDR_ALIGNMENT, input_size);
+	if (!tsk->input)
+		return -ENOMEM;
+	memset_pattern(tsk->input, tsk->pattern, input_size);
+
+	tsk->src1 = aligned_alloc(ADDR_ALIGNMENT, input_size);
+	if (!tsk->src1)
+		return -ENOMEM;
+	memset_pattern(tsk->src1, 0, input_size);
+
+	tsk->src2 = aligned_alloc(ADDR_ALIGNMENT, IAA_CRYPTO_SRC2_SIZE);
+	if (!tsk->src2)
+		return -ENOMEM;
+	memset_pattern(tsk->src2, 0, IAA_CRYPTO_SRC2_SIZE);
+	tsk->iaa_src2_xfer_size = IAA_CRYPTO_AECS_SIZE;
+	iaa_crypto_aecs = (struct iaa_crypto_aecs_t *)tsk->src2;
+	iaa_crypto_aecs->crypto_algorithm = tsk->crypto_aecs.algorithm;
+	iaa_crypto_aecs->crypto_flags = tsk->crypto_aecs.flags;
+
+	if (iaa_crypto_aecs->crypto_flags & IAA_CRYPTO_MASK_KEY_SIZE)
+		key_size = 256;
+	else
+		key_size = 128;
+
+	iaa_crypto_aecs->crypto_flags |= IAA_CRYPTO_MASK_FLUSH_CRYPTO_IN_ACCUM;
+
+	switch (iaa_crypto_aecs->crypto_algorithm) {
+	case IAA_AES_CFB:
+		if (key_size == 256) {
+			for (i = 0; i < 4; i++)
+				iaa_crypto_aecs->aes_key_low[i] = (uint32_t)get_random_value();
+			for (i = 0; i < 4; i++)
+				iaa_crypto_aecs->aes_key_high[i] = (uint32_t)get_random_value();
+			for (i = 0; i < 4; i++)
+				iaa_crypto_aecs->counter_iv[i] = (uint32_t)get_random_value();
+		} else {
+			for (i = 0; i < 4; i++)
+				iaa_crypto_aecs->aes_key_low[i] = (uint32_t)get_random_value();
+			for (i = 0; i < 4; i++)
+				iaa_crypto_aecs->counter_iv[i] = (uint32_t)get_random_value();
+		}
+		break;
+	}
+
+	iaa_crypto_aecs->complement[8] = 1;
+	tsk->xfer_size = iaa_do_crypto(tsk->src1, tsk->input, input_size,
+				       (uint8_t *)iaa_crypto_aecs->aes_key_low,
+				       (uint8_t *)iaa_crypto_aecs->counter_iv,
+				       key_size, iaa_crypto_aecs->crypto_algorithm, 1);
+
+	if (tsk->xfer_size != input_size) {
+		err("Pre encrypted size %d is not equal to input size %d\n",
+		    tsk->xfer_size, input_size);
+		return -ENOMEM;
+	}
+
+	tsk->dst1 = aligned_alloc(ADDR_ALIGNMENT, input_size);
+	if (!tsk->dst1)
+		return -ENOMEM;
+	memset_pattern(tsk->dst1, 0, input_size);
+
+	tsk->output = aligned_alloc(ADDR_ALIGNMENT, input_size);
+	if (!tsk->output)
+		return -ENOMEM;
+	memset_pattern(tsk->output, 0, input_size);
+
+	tsk->iaa_max_dst_size = input_size;
+
+	return ACCTEST_STATUS_OK;
+}
+
 int init_task(struct task *tsk, int tflags, int opcode, unsigned long src1_xfer_size)
 {
 	int rc = 0;
@@ -503,6 +734,12 @@ int init_task(struct task *tsk, int tflags, int opcode, unsigned long src1_xfer_
 	case IAX_OPCODE_CRC64: /* intentionally empty */
 		rc = init_crc64(tsk, tflags, opcode, src1_xfer_size);
 		break;
+	case IAX_OPCODE_ZCOMPRESS8:
+		rc = init_zcompress8(tsk, tflags, opcode, src1_xfer_size);
+		break;
+	case IAX_OPCODE_ZDECOMPRESS8:
+		rc = init_zdecompress8(tsk, tflags, opcode, src1_xfer_size);
+		break;
 	case IAX_OPCODE_ZCOMPRESS16:
 		rc = init_zcompress16(tsk, tflags, opcode, src1_xfer_size);
 		break;
@@ -542,6 +779,15 @@ int init_task(struct task *tsk, int tflags, int opcode, unsigned long src1_xfer_
 	case IAX_OPCODE_EXPAND:
 		rc = init_expand(tsk, tflags, opcode, src1_xfer_size);
 		break;
+	case IAX_OPCODE_TRANSL_FETCH:
+		rc = init_transl_fetch(tsk, tflags, opcode, src1_xfer_size);
+		break;
+	case IAX_OPCODE_ENCRYPT:
+		rc = init_encrypto(tsk, tflags, opcode, src1_xfer_size);
+		break;
+	case IAX_OPCODE_DECRYPT:
+		rc = init_decrypto(tsk, tflags, opcode, src1_xfer_size);
+		break;
 	}
 
 	if (rc != ACCTEST_STATUS_OK) {
@@ -646,6 +892,100 @@ int iaa_crc64_multi_task_nodes(struct acctest_context *ctx)
 	return ret;
 }
 
+static int iaa_wait_zcompress8(struct acctest_context *ctx, struct task *tsk)
+{
+	struct completion_record *comp = tsk->comp;
+	int rc;
+
+	rc = acctest_wait_on_desc_timeout(comp, ctx, ms_timeout);
+	if (rc < 0) {
+		err("zcompress8 desc timeout\n");
+		return ACCTEST_STATUS_TIMEOUT;
+	}
+
+	return ACCTEST_STATUS_OK;
+}
+
+int iaa_zcompress8_multi_task_nodes(struct acctest_context *ctx)
+{
+	struct task_node *tsk_node = ctx->multi_task_node;
+	int ret = ACCTEST_STATUS_OK;
+
+	while (tsk_node) {
+		tsk_node->tsk->dflags |= (IDXD_OP_FLAG_CRAV | IDXD_OP_FLAG_RCR);
+		if ((tsk_node->tsk->test_flags & TEST_FLAGS_BOF) && ctx->bof)
+			tsk_node->tsk->dflags |= IDXD_OP_FLAG_BOF;
+
+		iaa_prep_zcompress8(tsk_node->tsk);
+		tsk_node = tsk_node->next;
+	}
+
+	info("Submitted all zcompress8 jobs\n");
+	tsk_node = ctx->multi_task_node;
+	while (tsk_node) {
+		acctest_desc_submit(ctx, tsk_node->tsk->desc);
+		tsk_node = tsk_node->next;
+	}
+
+	tsk_node = ctx->multi_task_node;
+	while (tsk_node) {
+		ret = iaa_wait_zcompress8(ctx, tsk_node->tsk);
+		if (ret != ACCTEST_STATUS_OK)
+			info("Desc: %p failed with ret: %d\n",
+			     tsk_node->tsk->desc, tsk_node->tsk->comp->status);
+		tsk_node = tsk_node->next;
+	}
+
+	return ret;
+}
+
+static int iaa_wait_zdecompress8(struct acctest_context *ctx, struct task *tsk)
+{
+	struct completion_record *comp = tsk->comp;
+	int rc;
+
+	rc = acctest_wait_on_desc_timeout(comp, ctx, ms_timeout);
+	if (rc < 0) {
+		err("zdecompress8 desc timeout\n");
+		return ACCTEST_STATUS_TIMEOUT;
+	}
+
+	return ACCTEST_STATUS_OK;
+}
+
+int iaa_zdecompress8_multi_task_nodes(struct acctest_context *ctx)
+{
+	struct task_node *tsk_node = ctx->multi_task_node;
+	int ret = ACCTEST_STATUS_OK;
+
+	while (tsk_node) {
+		tsk_node->tsk->dflags |= (IDXD_OP_FLAG_CRAV | IDXD_OP_FLAG_RCR);
+		if ((tsk_node->tsk->test_flags & TEST_FLAGS_BOF) && ctx->bof)
+			tsk_node->tsk->dflags |= IDXD_OP_FLAG_BOF;
+
+		iaa_prep_zdecompress8(tsk_node->tsk);
+		tsk_node = tsk_node->next;
+	}
+
+	info("Submitted all zdecompress8 jobs\n");
+	tsk_node = ctx->multi_task_node;
+	while (tsk_node) {
+		acctest_desc_submit(ctx, tsk_node->tsk->desc);
+		tsk_node = tsk_node->next;
+	}
+
+	tsk_node = ctx->multi_task_node;
+	while (tsk_node) {
+		ret = iaa_wait_zdecompress8(ctx, tsk_node->tsk);
+		if (ret != ACCTEST_STATUS_OK)
+			info("Desc: %p failed with ret: %d\n",
+			     tsk_node->tsk->desc, tsk_node->tsk->comp->status);
+		tsk_node = tsk_node->next;
+	}
+
+	return ret;
+}
+
 static int iaa_wait_zcompress16(struct acctest_context *ctx, struct task *tsk)
 {
 	struct completion_record *comp = tsk->comp;
@@ -865,7 +1205,7 @@ int iaa_compress_multi_task_nodes(struct acctest_context *ctx)
 
 		tsk_node->tsk->iaa_compr_flags = (IDXD_COMPRESS_FLAG_EOB_BFINAL |
 						  IDXD_COMPRESS_FLAG_FLUSH_OUTPUT);
-		tsk_node->tsk->iaa_max_dst_size = ctx->wq_max_xfer_size;
+		tsk_node->tsk->iaa_max_dst_size = IAA_COMPRESS_MAX_DEST_SIZE;
 
 		iaa_prep_compress(tsk_node->tsk);
 		tsk_node = tsk_node->next;
@@ -923,7 +1263,7 @@ int iaa_decompress_multi_task_nodes(struct acctest_context *ctx)
 
 		tsk_node->tsk->iaa_compr_flags = (IDXD_COMPRESS_FLAG_EOB_BFINAL |
 						  IDXD_COMPRESS_FLAG_FLUSH_OUTPUT);
-		tsk_node->tsk->iaa_max_dst_size = ctx->wq_max_xfer_size;
+		tsk_node->tsk->iaa_max_dst_size = IAA_DECOMPRESS_MAX_DEST_SIZE;
 
 		iaa_prep_compress(tsk_node->tsk);
 		tsk_node = tsk_node->next;
@@ -974,7 +1314,7 @@ int iaa_decompress_multi_task_nodes(struct acctest_context *ctx)
 						    IDXD_DECOMPRESS_FLAG_STOP_ON_EOB |
 						    IDXD_DECOMPRESS_FLAG_FLUSH_OUTPUT |
 						    IDXD_DECOMPRESS_FLAG_EN_DECOMPRESS);
-		tsk_node->tsk->iaa_max_dst_size = ctx->wq_max_xfer_size;
+		tsk_node->tsk->iaa_max_dst_size = IAA_DECOMPRESS_MAX_DEST_SIZE;
 
 		iaa_prep_decompress(tsk_node->tsk);
 		tsk_node = tsk_node->next;
@@ -1340,20 +1680,177 @@ int iaa_expand_multi_task_nodes(struct acctest_context *ctx)
 	return ret;
 }
 
+static int iaa_wait_transl_fetch(struct acctest_context *ctx, struct task *tsk)
+{
+	struct completion_record *comp = tsk->comp;
+	int rc;
+
+	rc = acctest_wait_on_desc_timeout(comp, ctx, ms_timeout);
+	if (rc < 0) {
+		err("transl_fetch desc timeout\n");
+		return ACCTEST_STATUS_TIMEOUT;
+	}
+
+	return ACCTEST_STATUS_OK;
+}
+
+int iaa_transl_fetch_multi_task_nodes(struct acctest_context *ctx)
+{
+	struct task_node *tsk_node = ctx->multi_task_node;
+	int ret = ACCTEST_STATUS_OK;
+
+	while (tsk_node) {
+		tsk_node->tsk->dflags |= (IDXD_OP_FLAG_CRAV | IDXD_OP_FLAG_RCR);
+		if ((tsk_node->tsk->test_flags & TEST_FLAGS_BOF) && ctx->bof)
+			tsk_node->tsk->dflags |= IDXD_OP_FLAG_BOF;
+
+		iaa_prep_transl_fetch(tsk_node->tsk);
+		tsk_node = tsk_node->next;
+	}
+
+	info("Submitted all transl_fetch jobs\n");
+	tsk_node = ctx->multi_task_node;
+	while (tsk_node) {
+		acctest_desc_submit(ctx, tsk_node->tsk->desc);
+		tsk_node = tsk_node->next;
+	}
+
+	tsk_node = ctx->multi_task_node;
+	while (tsk_node) {
+		ret = iaa_wait_transl_fetch(ctx, tsk_node->tsk);
+		if (ret != ACCTEST_STATUS_OK)
+			info("Desc: %p failed with ret: %d\n",
+			     tsk_node->tsk->desc, tsk_node->tsk->comp->status);
+		tsk_node = tsk_node->next;
+	}
+
+	return ret;
+}
+
+static int iaa_wait_encrypto(struct acctest_context *ctx, struct task *tsk)
+{
+	struct completion_record *comp = tsk->comp;
+	int rc;
+
+	rc = acctest_wait_on_desc_timeout(comp, ctx, ms_timeout);
+	if (rc < 0) {
+		err("encrypto desc timeout\n");
+		return ACCTEST_STATUS_TIMEOUT;
+	}
+
+	return ACCTEST_STATUS_OK;
+}
+
+int iaa_encrypto_multi_task_nodes(struct acctest_context *ctx)
+{
+	struct task_node *tsk_node = ctx->multi_task_node;
+	int ret = ACCTEST_STATUS_OK;
+
+	while (tsk_node) {
+		tsk_node->tsk->dflags |= (IDXD_OP_FLAG_CRAV | IDXD_OP_FLAG_RCR);
+		if ((tsk_node->tsk->test_flags & TEST_FLAGS_BOF) && ctx->bof)
+			tsk_node->tsk->dflags |= IDXD_OP_FLAG_BOF;
+
+		tsk_node->tsk->dflags |= IDXD_OP_FLAG_RD_SRC2_AECS;
+		tsk_node->tsk->iaa_cipher_flags |= IDXD_CRYPTO_CIPHER_FLAG_FLUSH_OUTPUT;
+
+		iaa_prep_encrypto(tsk_node->tsk);
+		tsk_node = tsk_node->next;
+	}
+
+	info("Submitted all encrypto jobs\n");
+	tsk_node = ctx->multi_task_node;
+	while (tsk_node) {
+		acctest_desc_submit(ctx, tsk_node->tsk->desc);
+		dump_src2(tsk_node->tsk->src2, IAA_CRYPTO_AECS_SIZE);
+		tsk_node = tsk_node->next;
+	}
+
+	tsk_node = ctx->multi_task_node;
+	while (tsk_node) {
+		ret = iaa_wait_encrypto(ctx, tsk_node->tsk);
+		if (ret != ACCTEST_STATUS_OK)
+			info("Desc: %p failed with ret: %d\n",
+			     tsk_node->tsk->desc, tsk_node->tsk->comp->status);
+		tsk_node = tsk_node->next;
+	}
+
+	return ret;
+}
+
+static int iaa_wait_decrypto(struct acctest_context *ctx, struct task *tsk)
+{
+	struct completion_record *comp = tsk->comp;
+	int rc;
+
+	rc = acctest_wait_on_desc_timeout(comp, ctx, ms_timeout);
+	if (rc < 0) {
+		err("decrypto desc timeout\n");
+		return ACCTEST_STATUS_TIMEOUT;
+	}
+
+	return ACCTEST_STATUS_OK;
+}
+
+int iaa_decrypto_multi_task_nodes(struct acctest_context *ctx)
+{
+	struct task_node *tsk_node = ctx->multi_task_node;
+	int ret = ACCTEST_STATUS_OK;
+
+	while (tsk_node) {
+		tsk_node->tsk->dflags |= (IDXD_OP_FLAG_CRAV | IDXD_OP_FLAG_RCR);
+		if ((tsk_node->tsk->test_flags & TEST_FLAGS_BOF) && ctx->bof)
+			tsk_node->tsk->dflags |= IDXD_OP_FLAG_BOF;
+
+		tsk_node->tsk->dflags |= IDXD_OP_FLAG_RD_SRC2_AECS;
+
+		iaa_prep_decrypto(tsk_node->tsk);
+		tsk_node = tsk_node->next;
+	}
+
+	info("Submitted all decrypto jobs\n");
+	tsk_node = ctx->multi_task_node;
+	while (tsk_node) {
+		acctest_desc_submit(ctx, tsk_node->tsk->desc);
+		dump_src2(tsk_node->tsk->src2, IAA_CRYPTO_AECS_SIZE);
+		tsk_node = tsk_node->next;
+	}
+
+	tsk_node = ctx->multi_task_node;
+	while (tsk_node) {
+		ret = iaa_wait_decrypto(ctx, tsk_node->tsk);
+		if (ret != ACCTEST_STATUS_OK)
+			info("Desc: %p failed with ret: %d\n",
+			     tsk_node->tsk->desc, tsk_node->tsk->comp->status);
+		tsk_node = tsk_node->next;
+	}
+
+	return ret;
+}
+
 /* mismatch_expected: expect mismatched buffer with success status 0x1 */
 int iaa_task_result_verify(struct task *tsk, int mismatch_expected)
 {
-	int ret = ACCTEST_STATUS_OK;
+	int ret = ACCTEST_STATUS_FAIL;
 
 	info("verifying task result for %#lx\n", tsk);
 
-	if (tsk->comp->status != IAX_COMP_SUCCESS)
+	if (tsk->opcode != IAX_OPCODE_TRANSL_FETCH && tsk->comp->status != IAX_COMP_SUCCESS)
 		return tsk->comp->status;
 
 	switch (tsk->opcode) {
+	case IAX_OPCODE_NOOP:
+		ret = ACCTEST_STATUS_OK;
+		break;
 	case IAX_OPCODE_CRC64:
 		ret = task_result_verify_crc64(tsk, mismatch_expected);
 		break;
+	case IAX_OPCODE_ZCOMPRESS8:
+		ret = task_result_verify_zcompress8(tsk, mismatch_expected);
+		break;
+	case IAX_OPCODE_ZDECOMPRESS8:
+		ret = task_result_verify_zdecompress8(tsk, mismatch_expected);
+		break;
 	case IAX_OPCODE_ZCOMPRESS16:
 		ret = task_result_verify_zcompress16(tsk, mismatch_expected);
 		break;
@@ -1369,6 +1866,9 @@ int iaa_task_result_verify(struct task *tsk, int mismatch_expected)
 	case IAX_OPCODE_COMPRESS:
 		ret = task_result_verify_compress(tsk, mismatch_expected);
 		break;
+	case IAX_OPCODE_DECOMPRESS:
+		ret = task_result_verify_decompress(tsk, mismatch_expected);
+		break;
 	case IAX_OPCODE_SCAN:
 		ret = task_result_verify_scan(tsk, mismatch_expected);
 		break;
@@ -1390,6 +1890,15 @@ int iaa_task_result_verify(struct task *tsk, int mismatch_expected)
 	case IAX_OPCODE_EXPAND:
 		ret = task_result_verify_expand(tsk, mismatch_expected);
 		break;
+	case IAX_OPCODE_TRANSL_FETCH:
+		ret = task_result_verify_transl_fetch(tsk, mismatch_expected);
+		break;
+	case IAX_OPCODE_ENCRYPT:
+		ret = task_result_verify_encrypto(tsk, mismatch_expected);
+		break;
+	case IAX_OPCODE_DECRYPT:
+		ret = task_result_verify_decrypto(tsk, mismatch_expected);
+		break;
 	}
 
 	if (ret == ACCTEST_STATUS_OK)
@@ -1455,6 +1964,88 @@ int task_result_verify_crc64(struct task *tsk, int mismatch_expected)
 	return -ENXIO;
 }
 
+int task_result_verify_zcompress8(struct task *tsk, int mismatch_expected)
+{
+	int i;
+	int rc;
+	int expected_len;
+
+	if (mismatch_expected)
+		warn("invalid arg mismatch_expected for %d\n", tsk->opcode);
+
+	expected_len = iaa_do_zcompress8(tsk->output, tsk->src1, tsk->xfer_size);
+	rc = memcmp(tsk->dst1, tsk->output, expected_len);
+
+	if (!mismatch_expected) {
+		if (expected_len - tsk->comp->iax_output_size) {
+			err("zcompress8 mismatch, exp len %d, act len %d\n",
+			    expected_len, tsk->comp->iax_output_size);
+
+			return -ENXIO;
+		}
+		if (rc) {
+			err("zcompress8 mismatch, memcmp rc %d\n", rc);
+			for (i = 0; i < (expected_len); i++) {
+				printf("Exp[%d]=0x%02X, Act[%d]=0x%02X\n",
+				       i, ((uint8_t *)tsk->output)[i],
+				       i, ((uint8_t *)tsk->dst1)[i]);
+			}
+
+			return -ENXIO;
+		}
+		return ACCTEST_STATUS_OK;
+	}
+
+	/* mismatch_expected */
+	if (rc) {
+		info("expected mismatch\n");
+		return ACCTEST_STATUS_OK;
+	}
+
+	return -ENXIO;
+}
+
+int task_result_verify_zdecompress8(struct task *tsk, int mismatch_expected)
+{
+	int i;
+	int rc;
+	int expected_len;
+
+	if (mismatch_expected)
+		warn("invalid arg mismatch_expected for %d\n", tsk->opcode);
+
+	expected_len = iaa_do_zdecompress8(tsk->output, tsk->src1, tsk->xfer_size);
+	rc = memcmp(tsk->dst1, tsk->output, expected_len);
+
+	if (!mismatch_expected) {
+		if (expected_len - tsk->comp->iax_output_size) {
+			err("zdecompress8 mismatch, exp len %d, act len %d\n",
+			    expected_len, tsk->comp->iax_output_size);
+
+			return -ENXIO;
+		}
+		if (rc) {
+			err("zdecompress8 mismatch, memcmp rc %d\n", rc);
+			for (i = 0; i < (expected_len); i++) {
+				printf("Exp[%d]=0x%02X, Act[%d]=0x%02X\n",
+				       i, ((uint8_t *)tsk->output)[i],
+				       i, ((uint8_t *)tsk->dst1)[i]);
+			}
+
+			return -ENXIO;
+		}
+		return ACCTEST_STATUS_OK;
+	}
+
+	/* mismatch_expected */
+	if (rc) {
+		info("expected mismatch\n");
+		return ACCTEST_STATUS_OK;
+	}
+
+	return -ENXIO;
+}
+
 int task_result_verify_zcompress16(struct task *tsk, int mismatch_expected)
 {
 	int i;
@@ -1623,7 +2214,7 @@ int task_result_verify_compress(struct task *tsk, int mismatch_expected)
 {
 	int i = 0;
 	int rc;
-	int expected_len;
+	int expected_len = 0;
 
 	if (mismatch_expected)
 		warn("invalid arg mismatch_expected for %d\n", tsk->opcode);
@@ -1995,3 +2586,120 @@ int task_result_verify_expand(struct task *tsk, int mismatch_expected)
 	return -ENXIO;
 }
 
+int task_result_verify_transl_fetch(struct task *tsk, int mismatch_expected)
+{
+	int rc = ACCTEST_STATUS_OK;
+
+	if (mismatch_expected) {
+		if (tsk->comp->status == DSA_COMP_PAGE_FAULT_NOBOF) {
+			if (tsk->comp->fault_addr != (uint64_t)tsk->src1) {
+				err("fault addr=0x%llX, src1=0x%llX, xfer size=0x%llX\n",
+				    tsk->comp->fault_addr, tsk->src1, tsk->xfer_size);
+				rc = -EFAULT;
+			} else {
+				warn("mismatch_expected for %d\n", tsk->opcode);
+			}
+		} else {
+			err("error status code 0x%x\n", tsk->comp->status);
+			rc = -EINVAL;
+		}
+	}
+
+	return rc;
+}
+
+int task_result_verify_encrypto(struct task *tsk, int mismatch_expected)
+{
+	int rc, i, key_size, expected_len;
+	struct iaa_crypto_aecs_t *iaa_crypto_aecs = (struct iaa_crypto_aecs_t *)tsk->src2;
+
+	if (mismatch_expected)
+		warn("invalid arg mismatch_expected for %d\n", tsk->opcode);
+
+	if (iaa_crypto_aecs->crypto_flags & IAA_CRYPTO_MASK_KEY_SIZE)
+		key_size = 256;
+	else
+		key_size = 128;
+
+	expected_len = iaa_do_crypto(tsk->output, tsk->src1, tsk->xfer_size,
+				     (uint8_t *)iaa_crypto_aecs->aes_key_low,
+				     (uint8_t *)iaa_crypto_aecs->counter_iv,
+				     key_size, iaa_crypto_aecs->crypto_algorithm, 1);
+	rc = memcmp(tsk->dst1, tsk->output, expected_len);
+
+	if (!mismatch_expected) {
+		if (expected_len - tsk->comp->iax_output_size) {
+			err("encrypto mismatch, exp len %d, act len %d\n",
+			    expected_len, tsk->comp->iax_output_size);
+
+			return -ENXIO;
+		}
+		if (rc) {
+			err("encrypto mismatch, memcmp rc %d\n", rc);
+			for (i = 0; i < (expected_len); i++) {
+				printf("Exp[%d]=0x%02X, Act[%d]=0x%02X\n",
+				       i, ((uint8_t *)tsk->output)[i],
+				       i, ((uint8_t *)tsk->dst1)[i]);
+			}
+
+			return -ENXIO;
+		}
+		return ACCTEST_STATUS_OK;
+	}
+
+	/* mismatch_expected */
+	if (rc) {
+		info("expected mismatch\n");
+		return ACCTEST_STATUS_OK;
+	}
+
+	return -ENXIO;
+}
+
+int task_result_verify_decrypto(struct task *tsk, int mismatch_expected)
+{
+	int rc, i, key_size, expected_len;
+	struct iaa_crypto_aecs_t *iaa_crypto_aecs = (struct iaa_crypto_aecs_t *)tsk->src2;
+
+	if (mismatch_expected)
+		warn("invalid arg mismatch_expected for %d\n", tsk->opcode);
+
+	if (iaa_crypto_aecs->crypto_flags & IAA_CRYPTO_MASK_KEY_SIZE)
+		key_size = 256;
+	else
+		key_size = 128;
+
+	expected_len = iaa_do_crypto(tsk->output, tsk->src1, tsk->xfer_size,
+				     (uint8_t *)iaa_crypto_aecs->aes_key_low,
+				     (uint8_t *)iaa_crypto_aecs->counter_iv,
+				     key_size, iaa_crypto_aecs->crypto_algorithm, 0);
+	rc = memcmp(tsk->dst1, tsk->output, expected_len);
+
+	if (!mismatch_expected) {
+		if (expected_len - tsk->comp->iax_output_size) {
+			err("decrypto mismatch, exp len %d, act len %d\n",
+			    expected_len, tsk->comp->iax_output_size);
+
+			return -ENXIO;
+		}
+		if (rc) {
+			err("decrypto mismatch, memcmp rc %d\n", rc);
+			for (i = 0; i < (expected_len); i++) {
+				printf("Exp[%d]=0x%02X, Act[%d]=0x%02X\n",
+				       i, ((uint8_t *)tsk->output)[i],
+				       i, ((uint8_t *)tsk->dst1)[i]);
+			}
+
+			return -ENXIO;
+		}
+		return ACCTEST_STATUS_OK;
+	}
+
+	/* mismatch_expected */
+	if (rc) {
+		info("expected mismatch\n");
+		return ACCTEST_STATUS_OK;
+	}
+
+	return -ENXIO;
+}
